import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class loantime_reordered extends PApplet {

// rows: 100 to 2000 with step of 50
// columns: 0 to 900 with step of 100
int rows = 23, cols = 10;
int[][] data, orig_data;
int xMargin, yMargin;
int xLeftover, yLeftover;
int boxWidth, boxHeight, boxMargin;
int minTime, maxTime;
PFont charFont, numFont, credFont;
int[] gradient_curve;
int[] gradient_colorset;
boolean ADAPTIVE_COLORMAP;
boolean HIGHLIGHT;
int highlight_idxx = -1, highlight_idxy = -1;
int magicnum_translationx = 0, magicnum_translationy = -10;

public void setup() {
    size(1800, 900);
    noLoop();
    
    Table t = loadTable("hw3.csv", "header");
    int r = t.getRowCount();
    int c = t.getColumnCount(); 
    data = new int[rows+1][cols+1];
    orig_data = new int[rows+1][cols+1];
    for (int i = 0; i < rows; ++i) {
        for (int j = 0; j < cols; ++j) {
            data[i][j] = 0;
        }
    }
    // 0th col: pages, 1st col: dewey, 4th col: avg_time
    // pages start from 100 with increment of 50
    // dewey class start from 0 with increment of 100
    minTime = 123456789;
    maxTime = 0;
    int[] unordered_data = new int[r];
    int data_count = 0;
    for (int i = 0; i < r; ++i) {
        int p = t.getInt(i, 0);
        int d = t.getInt(i, 1);
        int l = t.getInt(i, 4);
        int x = p / 50 - 2;
        int y = d / 100;
        if (l > 0 && x < rows && y < cols) {
            data[x][y] = l;
            if (l > maxTime) maxTime = l;
            if (l < minTime) minTime = l;
            // get raw unordered data and use it for color gradient generation
            unordered_data[i] = l;
            ++data_count;
        }
    }
    // store pages and dewey num on the same column and row with data
    for (int i = 0; i < rows; ++i) {
        data[i][cols] = (i + 2) * 50;
    }
    for (int i = 0; i < cols + 1; ++i) {
        data[rows][i] = i * 100;
    }
    data[rows][cols] = 0;
    arrayCopy(data, orig_data);
    unordered_data = subset(unordered_data, 0, data_count);
    gradient_curve = gradientGen(unordered_data, minTime, maxTime);
    
    // automaticaaly setting cell sizes
    xMargin = 10 * width / 100;
    yMargin = 10 * height / 100;
    // rows from data used for x axis, cols for y.
    boxWidth = (width - 2 * xMargin) / (rows + 2);
    boxHeight = (height - 2 * yMargin) / (cols + 2);
    int widthTotal = boxWidth * (rows + 2) + 2 * xMargin;
    xLeftover = width - widthTotal;
    int heightTotal = boxHeight * (cols + 2) + 2 * yMargin;
    yLeftover = height - heightTotal;
    boxMargin = 2;
    
    numFont = loadFont("Monospaced-48.vlw");
    credFont = loadFont("Monospaced-12.vlw");
    charFont = loadFont("FrutigerLT-Light-32.vlw");
    
    gradient_colorset = new int[3];
    gradient_colorset[0] = color(0, 215, 120);
    gradient_colorset[1] = color(215, 215, 120);
    gradient_colorset[2] = color(215, 0, 120);

    ADAPTIVE_COLORMAP = false;
    HIGHLIGHT = false;
}

public void draw(){
    background(240);

    // title
    textFont(charFont);
    textAlign(LEFT, CENTER);
    textSize(32);
    
    pushMatrix();
    translate(0.5f * xMargin, 0.5f * yMargin);
    fill(50);
    text("Pages", 0.3f * xMargin, 0.05f * yMargin);
    
    // diagonal line
    pushMatrix();
    rotate(-PI / 4);
    stroke(100);
    strokeWeight(2);
    line(0, 90, 0, -10);
    popMatrix();
    
    pushMatrix();
    rotate(-PI / 2);
    textAlign(RIGHT, CENTER);
    text("Dewey Class", -0.3f * xMargin, 0);
    popMatrix();
    
    popMatrix();
    
    // credit
    pushMatrix();
    fill(150);
    textFont(credFont);
    translate(30, height - 20);
    textSize(12);
    textAlign(LEFT, BOTTOM);
    String s = "Keehong Youn. UCSB MAT 259 Winter 2015";
    text(s, 0, 0);
    popMatrix();
    
    // for all cell related things, translate below is base translation
    translate(xMargin, yMargin);
    translate(xLeftover/2, yLeftover/2);
    translate(magicnum_translationx, magicnum_translationy); // magic number
    
    // cells
    noStroke();
    int tr = maxTime - minTime;
    for (int i = 0; i < rows; ++i) {
    for (int j = 0; j < cols; ++j) {
        int x = (i + 1) * boxWidth + boxMargin;
        int y = (j + 1) * boxHeight + boxMargin;
        int w = boxWidth - boxMargin;
        int h = boxHeight - boxMargin;
        int t = data[i][j]; 
        if ( t > 0) {
        float p = (float)(t - minTime) / tr;
        if (!ADAPTIVE_COLORMAP) {
            fill(colorMap(p));
        } else {
            fill(getGradient(p, gradient_colorset, gradient_curve));
        }
        } else {
        fill(210);
        }
        rect(x, y, w, h);
    }
    }

    // cells highlight, only if cursor in cell area
    if (HIGHLIGHT &&
        highlight_idxx >= 0 && highlight_idxx < rows &&
        highlight_idxy >= 0 && highlight_idxy < cols) {
        int highlightx = (highlight_idxx + 1) * boxWidth;
        int highlighty = (highlight_idxy + 1) * boxHeight;
        pushStyle();
        noFill();
        strokeWeight(7);
        stroke(170, 220, 255);
        rect(highlightx, highlighty, boxWidth, boxHeight);
        textAlign(CENTER,CENTER);
        textFont(numFont);
        textSize(22);
        fill(50);
        pushMatrix();
        translate(highlightx + boxWidth / 2, highlighty + boxHeight / 2);
        text(Integer.toString(data[highlight_idxx][highlight_idxy]), 0, 0);
        popMatrix();
        popStyle();
    }
    
    // row avg
    for (int j = 0; j < cols; ++j) {
    int x = (rows + 1) * boxWidth + boxMargin * 10;
    int y = (j + 1) * boxHeight + boxMargin;
    // int w = boxWidth - boxMargin * 10;
    int w = 20; // magic number for current dataset
    int h = boxHeight - boxMargin;
    int sum = 0;
    int miss = 0;
    // count miss so that doesn't affect avg
    for (int i = 0; i < rows; ++i) {
        int t = data[i][j];
        if (t > 0) sum += t;
        else ++miss;
    }
    // dividing by rows because rows were pages (x-axis)
    float p = ((float)sum / (rows - miss) - minTime) / tr;
    if (!ADAPTIVE_COLORMAP) {
        fill(colorMap(p));
    } else {
        fill(getGradient(p, gradient_colorset, gradient_curve));
    }
    rect(x, y, w, h);
    }
    
    // col avg
    for (int i = 0; i < rows; ++i) {
    int x = (i + 1) * boxWidth + boxMargin;
    int y = (cols + 1) * boxHeight + boxMargin * 10;
    int w = boxWidth - boxMargin;
    int h = boxHeight - boxMargin * 22;
    int sum = 0;
    int miss = 0;
    for (int j = 0; j < cols; ++j) {
        int t = data[i][j];
        if (t > 0) sum += t;
        else ++miss;
    }
    float p = ((float)sum / (cols - miss) - minTime) / tr;
    if (!ADAPTIVE_COLORMAP) {
        fill(colorMap(p));
    } else {
        fill(getGradient(p, gradient_colorset, gradient_curve));
    }
    rect(x, y, w, h);
    }
    
    // pages
    fill(100);
    textAlign(LEFT, BOTTOM);
    textFont(numFont, 14);
    for (int i = 0; i < rows; ++i) {
        pushMatrix();
        translate((i + 1 + 0.5f) * boxWidth, 0.8f * boxHeight);
        rotate(-PI / 4);
        int p = (i + 2) * 50;
        p = data[i][cols];
        if (p % 250 == 0) {
            textSize(22);
            text(Integer.toString(p), 0, 0);
            textSize(14);
        } else {
            text(Integer.toString(p), 0, 0);
        }
        popMatrix();
    }
    
    // dewey numbers
    textAlign(RIGHT, CENTER);
    textSize(22);
    for (int i = 0; i < cols; ++i) {
        pushMatrix();
        translate(0.5f * boxWidth, (i + 1 + 0.5f) * boxHeight);
        int d = i * 100;
        d = data[rows][i];
        text(Integer.toString(d), 0, 0);
        popMatrix();
    }
    
    // legend
    pushMatrix();
    // to right-1, bottom (total cell: (rows + 2), (cols + 2))
    translate((rows + 1) * boxWidth, (cols + 2) * boxHeight);
    int legendWidth = width * 20 / 100;
    if (legendWidth % 11 != 0) {
    legendWidth -= legendWidth % 11;
    }
    int legendHeight = height / 100;
    translate(-legendWidth, legendHeight * 2);
    int w = legendWidth / 11;
    for (int i = 0; i < 11; ++i) {
    int x = legendWidth / 11 * i;
    float p = (float)i / 11;
    if (!ADAPTIVE_COLORMAP) {
        fill(colorMap(p));
    } else {
        fill(getGradient(p, gradient_colorset, gradient_curve));
    }
    rect(x, 0, w, legendHeight);
    }
    // and legend numbers
    textSize(14);
    textAlign(CENTER, TOP);
    fill(100);
    translate(legendWidth / 11 / 2, 0); // below the center of legend cells
    text(Integer.toString(minTime), 0, legendHeight * 1.5f);
    text(Integer.toString((minTime + maxTime) / 2), 5 * legendWidth / 11, legendHeight * 1.5f);
    text(Integer.toString(maxTime), 10 * legendWidth / 11, legendHeight * 1.5f);
    popMatrix();
}

public int colorMap(float p) {
    int maxVal = 215;
    if (p > 0.5f) {
        int r = maxVal;
        int g = maxVal - PApplet.parseInt((p - 0.5f) * 2 * maxVal);
        return color(r, g, 120);
    } else {
        int r = PApplet.parseInt(sin(PI * p) * maxVal);
        int g = maxVal;
        return color(r, g, 120);
    }
}

public void mouseMoved() {
    if (HIGHLIGHT) {
        // same translation values with cells display in draw
        int offsetx = xMargin + xLeftover / 2 + magicnum_translationx;
        int offsety = yMargin + yLeftover / 2 + magicnum_translationy;
        int cursorx = mouseX - offsetx;
        int cursory = mouseY - offsety;
        // get cell index, reverse the fomula in draw, ignore the margin
        // int x = (i + 1) * boxWidth + boxMargin;
        // int y = (j + 1) * boxHeight + boxMargin;
        int horiz_idx = cursorx / boxWidth - 1;
        int verti_idx = cursory / boxHeight - 1;

        if (horiz_idx >= 0 && horiz_idx < rows &&
            verti_idx >= 0 && verti_idx < cols) {
            // if inside cell area
            if (horiz_idx == highlight_idxx && verti_idx == highlight_idxy) {
                // if in same cell as before, do nothing
                println("doing nothing" + random(1));
                return;
            } else {
                highlight_idxx = horiz_idx;
                highlight_idxy = verti_idx;
                redraw();
                println("in area, redraw" + random(1));
                return;
            }
        } else {
            // outside the cell area
            if (highlight_idxx >= 0 && highlight_idxx < rows &&
                highlight_idxy >= 0 && highlight_idxy < cols) {
                // if highlicgt was inside cell area, redraw to erase
                highlight_idxx = -1;
                highlight_idxy = -1;
                println("erasing highlight, redraw" + random(1));
                redraw();
                return;
            } else {
                // else do nothing
                println("doing nothing" + random(1));
                return;
            }
        }
    }
}

public void keyReleased() {
    if (key == 's' || key == 'S') {
    saveFrame("loantime.png");
    println("saved: loantime.png");
    return;
    }
    if (key == 'a' || key == 'A') {
    ADAPTIVE_COLORMAP = !ADAPTIVE_COLORMAP;
    println("ADAPTIVE_COLORMAP:", ADAPTIVE_COLORMAP);
    redraw();
    return;
    }
    if (key == 'h' || key == 'H') {
    HIGHLIGHT = !HIGHLIGHT;
    println("HIGHLIGHT:", HIGHLIGHT);
    return;
    }
    if (key == 'o' || key == 'O') {
        reorder(data);
        redraw();
        return;
    }
    if (key == 'r' || key == 'R') {
        // reset
        arrayCopy(orig_data, data);
        redraw();
        return;
    }
}


public void reorder(int[][] data) {
    // sort pages
    Arrays.sort(data, new Comparator<int[]>() {
        @Override
        public int compare(int[] o1, int[] o2) {
            float sum1 = 0;
            for (int i = 0; i < o1.length - 1; ++i) {
                sum1 += o1[i];
            }
            sum1 = sum1 / (o1.length-1);

            float sum2 = 0;
            for (int i = 0; i < o2.length - 1; ++i) {
                sum2 += o2[i];
            }
            sum2 = sum2 / (o2.length-1);
            return Float.compare(sum1, sum2);
        }
    });
    // sort dewey
    int[][] tempdata = transposeArr(data);
    Arrays.sort(tempdata, new Comparator<int[]>() {
        @Override
        public int compare(int[] o1, int[] o2) {
            float sum1 = 0;
            for (int i = 0; i < o1.length - 1; ++i) {
                sum1 += o1[i];
            }
            sum1 = sum1 / (o1.length-1);

            float sum2 = 0;
            for (int i = 0; i < o2.length - 1; ++i) {
                sum2 += o2[i];
            }
            sum2 = sum2 / (o2.length-1);
            return Float.compare(sum1, sum2);
        }
    });
    arrayCopy(transposeArr(tempdata), data);
}

public int[][] transposeArr(int[][] arr) {
    int ilen = arr.length;
    int jlen = arr[0].length;
    int[][] newarr = new int[jlen][ilen];
    for (int i = 0; i < ilen; ++i) {
        for (int j = 0; j < jlen; ++j) {
            newarr[j][i] = arr[i][j];
        }
    }
    return newarr;
}
public int[] gradientGen(int[] arr) {
  // first find max and min
  int arr_len = arr.length;
  int min = 123456789, max = 0;
  for (int i = 0; i < arr_len; ++i) {
    if (arr[i] > max) {
      max = arr[i];
    }
    if (arr[i] < min) {
      min = arr[i];
    }
  }
  return gradientGen(arr, min, max);
}

public int[] gradientGen(int[] arr, int min, int max) {
  // count occurence of values
  int arr_len = arr.length;
  int[] count = new int[max - min + 1];
  int count_len = count.length;
  for (int i = 0; i < count_len; ++i) {
    count[i] = 0;
  }
  for (int i = 0; i < arr_len; ++i) {
    count[arr[i] - min] += 1;
  }
  // integral values and normalize to 255
  for (int i = 1; i < count_len; ++i) {
    count[i] += count[i-1];
  }
  for (int i = 0; i < count_len; ++i) {
    count[i] = Math.round((float)count[i] / arr_len * 255);
    // avg with flat curve to prevent extreme exaggeration
    count[i] = round(0.5f * (count[i] + 255 * (float)i / (count_len - 1)));
  }
  return count;
}

public int getGradient(float p, int[] colors, int[] curve) {
  // p: float btwn 0~1, relative position of data in min~max range
  // newp: p mapped by curve
  int curve_length = curve.length - 1;
  int curve_idx = floor(p * curve_length);
  float newp = curve[curve_idx] / 255.0f;
  // return color(curve[curve_idx]);

  int num_color = colors.length;
  int color_range_start = 0;
  for (int i = 1; i < num_color; ++i) {
    if (newp <= (float)i / (num_color - 1)) {
      color_range_start = i - 1;
      break;
    }
  }
  // k: relative position btwn color range start and end
  // k = (p - start) / range length
  float range_length = 1.0f / (num_color - 1);
  float k = (newp - color_range_start * range_length) / range_length;
  int c = lerpColor(colors[color_range_start], colors[color_range_start + 1], k);
  return c;
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "loantime_reordered" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
