// rows: 100 to 2000 with step of 50
// columns: 0 to 900 with step of 100
int rows = 23, cols = 10;
int[][] data, orig_data;
int xMargin, yMargin;
int xLeftover, yLeftover;
int boxWidth, boxHeight, boxMargin;
int minTime, maxTime;
PFont charFont, numFont, credFont;
int[] gradient_curve;
color[] gradient_colorset;
boolean ADAPTIVE_COLORMAP;
boolean HIGHLIGHT;
int highlight_idxx = -1, highlight_idxy = -1;
int magicnum_translationx = 0, magicnum_translationy = -10;

void setup() {
    size(1800, 900);
    noLoop();
    
    Table t = loadTable("hw3.csv", "header");
    int r = t.getRowCount();
    int c = t.getColumnCount(); 
    data = new int[rows+1][cols+1];
    orig_data = new int[rows+1][cols+1];
    for (int i = 0; i < rows; ++i) {
        for (int j = 0; j < cols; ++j) {
            data[i][j] = 0;
        }
    }
    // 0th col: pages, 1st col: dewey, 4th col: avg_time
    // pages start from 100 with increment of 50
    // dewey class start from 0 with increment of 100
    minTime = 123456789;
    maxTime = 0;
    int[] unordered_data = new int[r];
    int data_count = 0;
    for (int i = 0; i < r; ++i) {
        int p = t.getInt(i, 0);
        int d = t.getInt(i, 1);
        int l = t.getInt(i, 4);
        int x = p / 50 - 2;
        int y = d / 100;
        if (l > 0 && x < rows && y < cols) {
            data[x][y] = l;
            if (l > maxTime) maxTime = l;
            if (l < minTime) minTime = l;
            // get raw unordered data and use it for color gradient generation
            unordered_data[i] = l;
            ++data_count;
        }
    }
    // store pages and dewey num on the same column and row with data
    for (int i = 0; i < rows; ++i) {
        data[i][cols] = (i + 2) * 50;
    }
    for (int i = 0; i < cols + 1; ++i) {
        data[rows][i] = i * 100;
    }
    data[rows][cols] = 0;
    arrayCopy(data, orig_data);
    unordered_data = subset(unordered_data, 0, data_count);
    gradient_curve = gradientGen(unordered_data, minTime, maxTime);
    
    // automaticaaly setting cell sizes
    xMargin = 10 * width / 100;
    yMargin = 10 * height / 100;
    // rows from data used for x axis, cols for y.
    boxWidth = (width - 2 * xMargin) / (rows + 2);
    boxHeight = (height - 2 * yMargin) / (cols + 2);
    int widthTotal = boxWidth * (rows + 2) + 2 * xMargin;
    xLeftover = width - widthTotal;
    int heightTotal = boxHeight * (cols + 2) + 2 * yMargin;
    yLeftover = height - heightTotal;
    boxMargin = 2;
    
    numFont = loadFont("Monospaced-48.vlw");
    credFont = loadFont("Monospaced-12.vlw");
    charFont = loadFont("FrutigerLT-Light-32.vlw");
    
    gradient_colorset = new color[3];
    gradient_colorset[0] = color(0, 215, 120);
    gradient_colorset[1] = color(215, 215, 120);
    gradient_colorset[2] = color(215, 0, 120);

    ADAPTIVE_COLORMAP = false;
    HIGHLIGHT = false;
}

void draw(){
    background(240);

    // title
    textFont(charFont);
    textAlign(LEFT, CENTER);
    textSize(32);
    
    pushMatrix();
    translate(0.5 * xMargin, 0.5 * yMargin);
    fill(50);
    text("Pages", 0.3 * xMargin, 0.05 * yMargin);
    
    // diagonal line
    pushMatrix();
    rotate(-PI / 4);
    stroke(100);
    strokeWeight(2);
    line(0, 90, 0, -10);
    popMatrix();
    
    pushMatrix();
    rotate(-PI / 2);
    textAlign(RIGHT, CENTER);
    text("Dewey Class", -0.3 * xMargin, 0);
    popMatrix();
    
    popMatrix();
    
    // credit
    pushMatrix();
    fill(150);
    textFont(credFont);
    translate(30, height - 20);
    textSize(12);
    textAlign(LEFT, BOTTOM);
    String s = "Keehong Youn. UCSB MAT 259 Winter 2015";
    text(s, 0, 0);
    popMatrix();
    
    // for all cell related things, translate below is base translation
    translate(xMargin, yMargin);
    translate(xLeftover/2, yLeftover/2);
    translate(magicnum_translationx, magicnum_translationy); // magic number
    
    // cells
    noStroke();
    int tr = maxTime - minTime;
    for (int i = 0; i < rows; ++i) {
        for (int j = 0; j < cols; ++j) {
            int x = (i + 1) * boxWidth + boxMargin;
            int y = (j + 1) * boxHeight + boxMargin;
            int w = boxWidth - boxMargin;
            int h = boxHeight - boxMargin;
            int t = data[i][j]; 
            if ( t > 0) {
            float p = (float)(t - minTime) / tr;
            if (!ADAPTIVE_COLORMAP) {
                fill(colorMap(p));
            } else {
                fill(getGradient(p, gradient_colorset, gradient_curve));
            }
            } else {
            fill(210);
            }
            rect(x, y, w, h);
        }
    }

    // cells highlight, only if cursor in cell area
    if (HIGHLIGHT &&
        highlight_idxx >= 0 && highlight_idxx < rows &&
        highlight_idxy >= 0 && highlight_idxy < cols) {
        int highlightx = (highlight_idxx + 1) * boxWidth;
        int highlighty = (highlight_idxy + 1) * boxHeight;
        pushStyle();
        noFill();
        strokeWeight(7);
        stroke(170, 220, 255);
        rect(highlightx, highlighty, boxWidth, boxHeight);
        textAlign(CENTER,CENTER);
        textFont(numFont);
        textSize(22);
        fill(50);
        pushMatrix();
        translate(highlightx + boxWidth / 2, highlighty + boxHeight / 2);
        text(Integer.toString(data[highlight_idxx][highlight_idxy]), 0, 0);
        popMatrix();
        popStyle();
    }
    
    // row avg
    for (int j = 0; j < cols; ++j) {
    int x = (rows + 1) * boxWidth + boxMargin * 10;
    int y = (j + 1) * boxHeight + boxMargin;
    // int w = boxWidth - boxMargin * 10;
    int w = 20; // magic number for current dataset
    int h = boxHeight - boxMargin;
    int sum = 0;
    int miss = 0;
    // count miss so that doesn't affect avg
    for (int i = 0; i < rows; ++i) {
        int t = data[i][j];
        if (t > 0) sum += t;
        else ++miss;
    }
    // dividing by rows because rows were pages (x-axis)
    float p = ((float)sum / (rows - miss) - minTime) / tr;
    if (!ADAPTIVE_COLORMAP) {
        fill(colorMap(p));
    } else {
        fill(getGradient(p, gradient_colorset, gradient_curve));
    }
    rect(x, y, w, h);
    }
    
    // col avg
    for (int i = 0; i < rows; ++i) {
    int x = (i + 1) * boxWidth + boxMargin;
    int y = (cols + 1) * boxHeight + boxMargin * 10;
    int w = boxWidth - boxMargin;
    int h = boxHeight - boxMargin * 22;
    int sum = 0;
    int miss = 0;
    for (int j = 0; j < cols; ++j) {
        int t = data[i][j];
        if (t > 0) sum += t;
        else ++miss;
    }
    float p = ((float)sum / (cols - miss) - minTime) / tr;
    if (!ADAPTIVE_COLORMAP) {
        fill(colorMap(p));
    } else {
        fill(getGradient(p, gradient_colorset, gradient_curve));
    }
    rect(x, y, w, h);
    }
    
    // pages
    fill(100);
    textAlign(LEFT, BOTTOM);
    textFont(numFont, 14);
    for (int i = 0; i < rows; ++i) {
        pushMatrix();
        translate((i + 1 + 0.5) * boxWidth, 0.8 * boxHeight);
        rotate(-PI / 4);
        int p = (i + 2) * 50;
        p = data[i][cols];
        if (p % 250 == 0) {
            textSize(22);
            text(Integer.toString(p), 0, 0);
            textSize(14);
        } else {
            text(Integer.toString(p), 0, 0);
        }
        popMatrix();
    }
    
    // dewey numbers
    textAlign(RIGHT, CENTER);
    textSize(22);
    for (int i = 0; i < cols; ++i) {
        pushMatrix();
        translate(0.5 * boxWidth, (i + 1 + 0.5) * boxHeight);
        int d = i * 100;
        d = data[rows][i];
        text(Integer.toString(d), 0, 0);
        popMatrix();
    }
    
    // legend
    pushMatrix();
    // to right-1, bottom (total cell: (rows + 2), (cols + 2))
    translate((rows + 1) * boxWidth, (cols + 2) * boxHeight);
    int legendWidth = width * 20 / 100;
    if (legendWidth % 11 != 0) {
        legendWidth -= legendWidth % 11;
    }
    int legendHeight = height / 100;
    translate(-legendWidth, legendHeight * 2);
    int w = legendWidth / 11;
    for (int i = 0; i < 11; ++i) {
        int x = legendWidth / 11 * i;
        float p = (float)i / 11;
        if (!ADAPTIVE_COLORMAP) {
            fill(colorMap(p));
        } else {
            fill(getGradient(p, gradient_colorset, gradient_curve));
        }
        rect(x, 0, w, legendHeight);
    }
    // and legend numbers
    textSize(14);
    textAlign(CENTER, TOP);
    fill(100);
    translate(legendWidth / 11 / 2, 0); // below the center of legend cells
    text(Integer.toString(minTime), 0, legendHeight * 1.5);
    text(Integer.toString((minTime + maxTime) / 2), 5 * legendWidth / 11, legendHeight * 1.5);
    text(Integer.toString(maxTime), 10 * legendWidth / 11, legendHeight * 1.5);
    popMatrix();
}

void mouseMoved() {
    if (HIGHLIGHT) {
        // same translation values with cells display in draw
        int offsetx = xMargin + xLeftover / 2 + magicnum_translationx;
        int offsety = yMargin + yLeftover / 2 + magicnum_translationy;
        int cursorx = mouseX - offsetx;
        int cursory = mouseY - offsety;
        // get cell index, reverse the fomula in draw, ignore the margin
        // int x = (i + 1) * boxWidth + boxMargin;
        // int y = (j + 1) * boxHeight + boxMargin;
        int horiz_idx = cursorx / boxWidth - 1;
        int verti_idx = cursory / boxHeight - 1;

        if (horiz_idx >= 0 && horiz_idx < rows &&
            verti_idx >= 0 && verti_idx < cols) {
            // if inside cell area
            if (horiz_idx == highlight_idxx && verti_idx == highlight_idxy) {
                // if in same cell as before, do nothing
                println("doing nothing" + random(1));
                return;
            } else {
                highlight_idxx = horiz_idx;
                highlight_idxy = verti_idx;
                redraw();
                println("in area, redraw" + random(1));
                return;
            }
        } else {
            // outside the cell area
            if (highlight_idxx >= 0 && highlight_idxx < rows &&
                highlight_idxy >= 0 && highlight_idxy < cols) {
                // if highlicgt was inside cell area, redraw to erase
                highlight_idxx = -1;
                highlight_idxy = -1;
                println("erasing highlight, redraw" + random(1));
                redraw();
                return;
            } else {
                // else do nothing
                println("doing nothing" + random(1));
                return;
            }
        }
    }
}
